import { Test, TestingModule } from '@nestjs/testing';
import {
  ConfigurationDomain,
  ConfigurationType,
} from '@modules/configuration/core/domain/configuration.types';
import {
  CONFIGURATION_MESSAGE_PUBLISHER,
  CONFIGURATION_REPOSITORY,
} from '@modules/configuration/configuration.di-tokens';
import { ConfigurationEntity } from '@modules/configuration/core/domain/configuration.entity';
import { PropagateConfigurationsService } from '@modules/configuration/core/application/commands/propagate-configurations/propagate-configurations.service';
import { CONFIGURATION_PROPAGATED_ROUTING_KEY } from '@src/app.constants';
import { ConfigurationMapper } from '@modules/configuration/configuration.mapper';

const mockMessagePublisher = {
  publish: jest.fn().mockImplementation(),
};

const configurationEntities = [
  new ConfigurationEntity({
    id: '047a6ecf-23d4-4d3e-877c-3225d560a8da',
    props: {
      identifier: {
        domain: ConfigurationDomain.CARPOOL,
        key: 'seatsProposed',
      },
      value: '3',
      type: ConfigurationType.NUMBER,
    },
    createdAt: new Date('2023-10-23T07:00:00Z'),
    updatedAt: new Date('2023-10-23T07:00:00Z'),
  }),
  new ConfigurationEntity({
    id: '047a6ecf-23d4-4d3e-877c-3225d560a8db',
    props: {
      identifier: {
        domain: ConfigurationDomain.CARPOOL,
        key: 'seatsRequested',
      },
      value: '1',
      type: ConfigurationType.NUMBER,
    },
    createdAt: new Date('2023-10-23T07:00:00Z'),
    updatedAt: new Date('2023-10-23T07:00:00Z'),
  }),
];

const mockConfigurationMapper = {
  toResponse: jest
    .fn()
    .mockImplementationOnce(() => ({
      domain: ConfigurationDomain.CARPOOL,
      key: 'seatsProposed',
      value: '3',
      type: ConfigurationType.NUMBER,
    }))
    .mockImplementationOnce(() => ({
      domain: ConfigurationDomain.CARPOOL,
      key: 'seatsRequested',
      value: '1',
      type: ConfigurationType.NUMBER,
    })),
};

const mockConfigurationRepository = {
  findAll: jest.fn().mockImplementationOnce(() => configurationEntities),
};

describe('Propagate Configurations Service', () => {
  let propagateConfigurationsService: PropagateConfigurationsService;

  beforeAll(async () => {
    const module: TestingModule = await Test.createTestingModule({
      providers: [
        {
          provide: CONFIGURATION_REPOSITORY,
          useValue: mockConfigurationRepository,
        },
        {
          provide: CONFIGURATION_MESSAGE_PUBLISHER,
          useValue: mockMessagePublisher,
        },
        {
          provide: ConfigurationMapper,
          useValue: mockConfigurationMapper,
        },
        PropagateConfigurationsService,
      ],
    }).compile();

    propagateConfigurationsService = module.get<PropagateConfigurationsService>(
      PropagateConfigurationsService,
    );
  });

  it('should be defined', () => {
    expect(propagateConfigurationsService).toBeDefined();
  });

  describe('execution', () => {
    it('should propagate configuration items', async () => {
      jest.spyOn(mockMessagePublisher, 'publish');
      await propagateConfigurationsService.execute();
      expect(mockMessagePublisher.publish).toHaveBeenCalledTimes(1);
      expect(mockMessagePublisher.publish).toHaveBeenCalledWith(
        CONFIGURATION_PROPAGATED_ROUTING_KEY,
        '[{"domain":"CARPOOL","key":"seatsProposed","value":"3","type":"NUMBER"},{"domain":"CARPOOL","key":"seatsRequested","value":"1","type":"NUMBER"}]',
      );
    });
  });
});
